/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function getBookName(aBook) {
	//crappy !!!
	if (aBook == 0)
		return "users";
	return "contacts";
}

function ObmAddressBookProxy() {
  //Init logger
  this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
  var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
  this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
  this.logger.setHeader("obmAddressBookProxy: ");
  this.logger.debug("new obmAddressBookProxy()", false, false);

	this.SOAPClient = Cc["@guru4.net/soapclient;1"].getService().wrappedJSObject;

	this._listeners = [];
	this._url = null;
	this._token = {
		user: null,
		domain: null,
		session_id: null
	};
};

ObmAddressBookProxy.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIAddressBookProxy) &&
			!aIID.equals(Ci.obmIProxyBase)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	/*
	 * obmIProxyBase
	 */
	get token() {
		return this._token;
	},
	set token(aToken) {
		this._token = aToken;
	},

	get WSDLUrl() {
		return this._url;
	},
	set WSDLUrl(aURL) {
		this._url = aURL;
	},


	addListener: function(listener) {
		for (var i = 0; i < this._listeners.length; ++i) {
			if (this._listeners[i] == listener)
				return;
		}
		this._listeners.push(listener);
	},

	removeListener: function(listener) {
		this._listeners = [];
	},
	_notifyListeners: function(aResultCode, aMethodName, aObj, aData) {
		// A copy is needed, a new listeners could has been added during the callbacks
		var listeners = this._listeners.slice(0, this._listeners.length);
		for (var i = 0; i < listeners.length; ++i) {
			try {
				//timer.(function() {
				listeners[i].onResponse(aResultCode, aMethodName, aObj, aData);
				//}, 1, timer.TYPE_ONE_SHOT);
			} catch(e) {
				//Continue to notify listeners
        this.logger.error(e + " (ignored)");
			}
		}
	},




	/*
	 * obmIAddressBookProxy
	 */
	_callback: function(aMethodName, aObj, aXml, aData) {
    this.logger.debug("_callback(aMethodName = " + aMethodName + ")", false, false);
		if (aObj && aObj.name == "Error") {
			var msg = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
			msg.data = aObj.fileName;
			this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, msg, aData);
			return;
		}
		switch(aMethodName) {
			case "logUserIn":
        if (!aObj.token || !aObj.token[0]) {
          var msg = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
          msg.data = "No token available";
          this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, msg, aData);
          return;
        }
				this._token.user = aObj.token[0].user[0];
				this._token.domain = aObj.token[0].domain[0];
				this._token.session_id = aObj.token[0].session_id[0];
				this._notifyListeners(Cr.NS_OK, aMethodName, null, aData);
				break;
			case "getSync":
				this._notifyListeners(Cr.NS_OK, aMethodName, new GetSyncContactResponseShell(aObj._return[0]), aData);
				break;
			case "getContactFromId":
			case "createContact":
			case "createContactWithoutDuplicate":
			case "modifyContact":
			case "removeContact":
				this._notifyListeners(Cr.NS_OK, aMethodName, new ContactShell(aObj._return[0]), aData);
				break;
			default:
			this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, null, aData);
		};
	},

	getSync: function(aBook, aLastSync, aData) {
    this.logger.debug("getSync()", false, false);
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("book", getBookName(aBook));
		p.add("lastsync", aLastSync);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "getSync", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	_createContactInternal: function(aBook, aContact, aUniq, aData) {
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("book", getBookName(aBook));
		p.add("contact", aContact);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, aUniq ? "createContactWithoutDuplicate" : "createContact", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	createContact: function(aBook, aContact, aData) {
    this.logger.debug("createContact()", false, false);
		return this._createContactInternal(aBook, aContact, false, aData);
	},
	createContactWithoutDuplicate: function(aBook, aContact, aData) {
    this.logger.debug("createContactWithoutDuplicate()", false, false);
		return this._createContactInternal(aBook, aContact, true, aData);
	},
	modifyContact: function(aBook, aContact, aData) {
    this.logger.debug("modifyContact()", false, false);
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("book", getBookName(aBook));
		p.add("contact", aContact);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "modifyContact", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	removeContact: function(aBook, aUid, aData) {
    this.logger.debug("removeContact()", false, false);
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("book", getBookName(aBook));
		p.add("id", aUid);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "removeContact", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	listBooks: function(aData) {
		throw Cr.NS_ERROR_NOT_IMPLEMENTED;
	},
	getContactFromId: function(aBook, aUid, aData) {
    this.logger.debug("getContactFromId()", false, false);
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("book", getBookName(aBook));
		p.add("id", aUid);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "getContactFromId", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	getContactTwinKeys: function (aBook, aContact, aData) {
		throw Cr.NS_ERROR_NOT_IMPLEMENTED;
	},
	logUserIn: function(aLogin, aPwd, aData) {
    this.logger.debug("logUserIn()", false, false);
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("login", aLogin);
		p.add("password", aPwd);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "logUserIn", p, true, function(name, obj, xml, data) {self._callback(name, obj, xml, data);}, aData);
	},
	isReadOnly: function(aBook, aData) {
		throw Cr.NS_ERROR_NOT_IMPLEMENTED;
	}
};

function ContactShell(inner) {
	this._mInner = inner;
}

ContactShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIContact)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	get uid() {return this._mInner.uid},
	get owner() {return this._mInner.owner},
	get classification() {return this._mInner.classification},
	get	firstName() {return this._mInner.firstName},
	get	lastName() {return this._mInner.lastName},
	get	email() {return this._mInner.email},
	get	email2() {return this._mInner.email2},
	get	workPhone() {return this._mInner.workPhone},
	get	mobilePhone() {return this._mInner.mobilePhone},
	get	homePhone() {return this._mInner.homePhone},
	get	address1() {return this._mInner.address1},
	get	address2() {return this._mInner.address2},
	get	address3() {return this._mInner.address3},
	get	zipCode() {return this._mInner.zipCode},
	get	town() {return this._mInner.town},
	get	expressPostal() {return this._mInner.expressPostal},
	get	country() {return this._mInner.country},
	get	company() {return this._mInner.company},
	get	title() {return this._mInner.title},
	get	workFax() {return this._mInner.workFax},
	get	homeFax() {return this._mInner.homeFax},
	get	comment() {return this._mInner.comment},
	get	service() {return this._mInner.service},
	get	aka() {return this._mInner.aka},
};

function GetSyncContactResponseShell(inner) {
	this._mInner = inner;
	this._updated = [];
	this._removed = [];


	if (inner.updated) for (var i = 0; i < inner.updated.length; i++) {
		this._updated.push(new ContactShell(inner.updated[i]));
	}
	if (inner.removed) for (var i = 0; i < inner.removed.length; i++) {
		this._removed.push(inner.removed[i]);
	}
}

GetSyncContactResponseShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmIAddressBookSyncResponse)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	getUpdatedContacts: function(count) {
		count.value = this._updated.length;
		return this._updated;
	},
	getRemovedContactIds: function(count) {
		count.value = this._removed.length;
		return this._removed;
	},
	get lastsync() {
		return this._mInner.lastsync[0];
	}
};


// module

var ObmAddressBookProxyModule = {
	mCID: Components.ID("{4b6693ea-1f04-4f0a-b133-0ff75c623640}"),
	mContractID: "@obm.org/obm-sync/addressbookproxy;1",
	registerSelf: function (compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(
			this.mCID,
			"OBM Address Book Proxy",
			this.mContractID,
			fileSpec,
			location,
			type
		);
	},
	getClassObject: function (compMgr, cid, iid) {
		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}
		return this.mFactory;
	},
	mFactory: {
		createInstance: function (outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmAddressBookProxy()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmAddressBookProxyModule;
}

