/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const RDFService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);

function ObmCalendarSync() {
	// Init logger
	this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
	var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
	this.stringBundle = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService).createBundle("chrome://obm-extension/locale/calendar/errors_warnings.properties");
	this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
	this.logger.setHeader("obmCalendarSync: ");
	this.logger.debug("new obmCalendarSync()", false, false);

	this.criticalMethods = ["logUserIn", "getSync"];

	this._timezone = calendarDefaultTimezone();
	this._localDiff = {};
	this._remoteDiff = {};
	
	var supportAlert = UtilsOBM.getBoolPref("extensions.obm.supportAlert", true);
	this._supportAlert = supportAlert;
};

function Context() {
	this.wrappedJSObject = this;
};

ObmCalendarSync.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) && !aIID.equals(Ci.obmICalendarSync)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	/*
	 * Private
	 */

	//
	// Helpers
	//
	_getLocalizedMessage: function(aString) {
		//dump(""+aString+"\n");
		try {
			return this.stringBundle.GetStringFromName("obm.syncerror." + aString);
		} catch(e) {
			return aString;
		}
	},
	_isCriticalMethod: function(aMethodeName) {
		for each(method in this.criticalMethods) {
			if (aMethodeName == method) {
				return true;
			}
		}
		return false;
	},
	_printableError: function(aMethodName, aMsg) {
		var printError = "srvError";

		if (aMsg == "No token available") {
			printError = "authFault";
		}

		if (aMsg == "Connection failure") {
			printError = "connectFail";
		}

		return this._getLocalizedMessage(printError);
	},
	_getOBMCredentials: function(aLogin, aPassword) {

		if (!UtilsOBM.getOBMCredentials(aLogin, aPassword) || !aLogin.value || aLogin.value == "" || !aPassword.value || aPassword.value == "") {
			var error = "NO_LOGIN_INFO";
			this.logger.error(error, true, false);
			this.callerListener.onError(error);
			throw (error);
		}
	},
	_getXPStringFromJSString: function(str) {
		var XPString = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
		XPString.data = str;
		return XPString;
	},
	_getJSStringFromXPString: function(XPString) {
		XPString.QueryInterface(Ci.nsISupportsString);
		return XPString.data;
	},

	_getLastSync: function(calendar) {
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		var calName = calMgr.getCalendarPref_(calendar, "X-OBM-NAME");
		var lastSync = UtilsOBM.getCharPref("extensions.obm.changes." + Ci.calICalendar + "." + calName + "." + "lastSync", "1970-01-01T00:00:00Z");
		if (!UtilsOBM.isValideDate(lastSync)) {
			lastSync = "1970-01-01T00:00:00Z";
		}
		return lastSync;
	},
	_setLastSync: function(calName, lastSync) {
		UtilsOBM.setCharPref("extensions.obm.changes." + Ci.calICalendar + "." + calName + "." + "lastSync", lastSync);
	},
	_createEventObjFromId: function(cal, aId, aWillBeCreated) {
		this.logger.debug("_createEventObjFromId()", false, false);
		var listener = {};
		var self = this;
		listener.calendar = cal;
		listener.willBeCreated = aWillBeCreated;
		listener.onOperationComplete = function(aCalendar, aStatus, aOperationType, aId, aDetail) {
			// <TO_REMOVE>
			// Fix second onOperationComplete call (bug LT 0.8)
			// Fixed in LT 0.9
			try {
				if (aDetail.toString() == "Can't deduce item type based on QI") {
					return;
				}
			} catch(e) {}
			// </TO_REMOVE>
			if (Components.isSuccessCode(aStatus)) {
				var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
				var calName = calMgr.getCalendarPref_(this.calendar, "X-OBM-NAME");
				var localEvent = this.localEvent;
				var eventToSend = {
					recurrence: this
				};
				eventToSend.QueryInterface = function(aIID) {
					if (!aIID.equals(Ci.nsISupports) && !aIID.equals(Ci.obmIEventRecurrence) && !aIID.equals(Ci.obmIEvent)) {
						throw Cr.NS_ERROR_NO_INTERFACE;
					}
					return this;
				};
				if (!this.willBeCreated) eventToSend.uid = localEvent.id;
				eventToSend.owner = null;
				var calPriority = localEvent.priority;

				if (calPriority >= 1 && calPriority <= 4) {
					eventToSend.priority = 3; // high priority in OBM
				} else if (calPriority == 5) {
					eventToSend.priority = 2; // medium priority in OBM
				} else if (calPriority >= 6 && calPriority <= 9) {
					eventToSend.priority = 1; // low priority in OBM
				} else {
					eventToSend.priority = 2; // medium priority in OBM, by default
				}

				switch (localEvent.privacy) {
				case "CONFIDENTIAL":
				case "PRIVATE":
					eventToSend.classification = "1";
					break;
				case "PUBLIC":
				default:
					eventToSend.classification = "0";
				}
				// 0 = public
				// 1 = private
				// ********************** //
				if (localEvent.startDate.isDate) {
					var begin = localEvent.startDate.jsDate;
					begin.setHours(8);
					eventToSend.allday = true;
					eventToSend.date = UtilsOBM.jsDateToXMLDate(begin);
					if (localEvent.duration.inSeconds >= 86400) {
						eventToSend.duration = localEvent.duration.inSeconds - 86400;
					} else {
						eventToSend.duration = localEvent.duration.inSeconds;
					}
				} else {
					eventToSend.allday = false;
					eventToSend.date = UtilsOBM.jsDateToXMLDate(localEvent.startDate.jsDate);
					eventToSend.duration = localEvent.duration.inSeconds;
				}
				eventToSend.title = localEvent.title;
				eventToSend.description = localEvent.getProperty("DESCRIPTION");

				if (localEvent.getCategories) {
					// lightning 0.9 support
					var category = localEvent.getCategories({}).toString();
				} else {
					var category = localEvent.getProperty("CATEGORIES");
				}
				if (category) {
					var matches = category.match(/\[OBM\] (.*)/);
					if (matches) {
						eventToSend.category = matches[1];
					}
				}

				eventToSend.location = localEvent.getProperty("LOCATION");

				self._mozRecurrenceToOBMEvent(localEvent, eventToSend);

				var allAttendees = localEvent.getAttendees({});
				var ownerEmail = calMgr.getCalendarPref_(this.calendar, "X-OBM-EMAIL");
				var calAttendee = localEvent.getAttendeeById("MAILTO:" + ownerEmail);
				var subscribedAdded = false;
				if (calAttendee) {
					// calAttendee object is immutable (read-only)
					// -> modifying a clone object
					var calAttendeeNew = calAttendee.clone();
					switch (localEvent.status) {
					case "CONFIRMED":
						calAttendeeNew.participationStatus = "ACCEPTED";
						break;
					case "CANCELLED":
						calAttendeeNew.participationStatus = "DECLINED";
						break;
					case "TENTATIVE":
						calAttendeeNew.participationStatus = "NEEDS-ACTION";
						break;
					}
					for (var i = 0; i < allAttendees.length; i++) {
						if (allAttendees[i].id == calAttendeeNew.id) {
							allAttendees.splice(i, 1, calAttendeeNew);
							break;
						}
					}
					subscribedAdded = true;
				}

				eventToSend.attendees = [];
				for each(var calAttendee in allAttendees) {
					var attendee = {};
					attendee.email = calAttendee.id.replace(/MAILTO:/i, "");
					attendee.displayName = calAttendee.commonName;
					switch (calAttendee.participationStatus) {
					case "ACCEPTED":
						attendee.status = "ACCEPTED";
						break;
					case "DECLINED":
						attendee.status = "REFUSED";
						break;
					case "NEEDS-ACTION":
					default:
						attendee.status = "WAITING";
						break;
					}
					switch (calAttendee.role) {
					case "REQ-PARTICIPANT":
						attendee.type = "REQUIRED";
						break;
					case "OPT-PARTICIPANT":
					default:
						attendee.type = "OPTIONAL";
						break;
					}
					eventToSend.attendees.push(attendee);
				}
				if (!subscribedAdded) {
					var attendee = {};
					attendee.email = ownerEmail;
					attendee.displayName = ownerEmail;
					attendee.status = "ACCEPTED";
					attendee.type = "REQUIRED";
					eventToSend.attendees.push(attendee);
				}

				eventToSend.getAttendees = function(count) {
					count.value = this.attendees.length;
					return this.attendees;
				};
				
				if (self._supportAlert) {
					if (localEvent.alarmOffset === undefined) {
						var alarms = localEvent.getAlarms({});
						for each (var alarm in alarms) {
							if (alarm.related == Ci.calIAlarm.ALARM_RELATED_START) {
								var calAlert = alarm.offset;
								break;
							}
						}
					} else {
						calAlert = localEvent.alarmOffset;
					}
	
					var calAlertToXmlAlert = function(aCalAlert) {
						if (aCalAlert.inSeconds > 0) {
							return 0; //OBM do not support alert after events
						} else {
							return ( - 1 * aCalAlert.inSeconds);
						}
					};
					
					var obmAlert = -1;
					if (calAlert) {
						if (localEvent.alarmLastAck) {
							if (localEvent.recurrenceInfo) {
								obmAlert = calAlertToXmlAlert(calAlert);
							}
						} else {
							obmAlert = calAlertToXmlAlert(calAlert);
						}
					}
					eventToSend.alert = obmAlert;
				} else {
					eventToSend.alert = -2;
				}

				var ctx = new Context();
				ctx.calendar = this.calendar;
				ctx.calName = calName;
				ctx.id = aId;
				ctx.localEvent = localEvent;
				ctx.eventToSend = eventToSend;
				if (this.willBeCreated) {
					ctx.methodName = "createEvent";
					self.CalProxy.createEvent(calName, eventToSend, ctx);
				} else {
					ctx.methodName = "modifyEvent";
					self.CalProxy.modifyEvent(calName, eventToSend, true, ctx);
				}

				UtilsOBM.removeItemFromChangesList(Ci.calICalendar, calName, Ci.calIEvent, this.willBeCreated ? "added": "updated", aId);
			}
		};
		listener.onGetResult = function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
			if (aCount > 0) {
				this.localEvent = aItems[0];
			}
		};

		cal.getItem(aId, listener);
	},

	_mozRecurrenceToOBMEvent: function(aCalEvent, aObmEvent) {
		var recurrenceInfo = aCalEvent.recurrenceInfo;
		var exceptionIDs = [];
		var invalidChoice;
		var endDate = new Date();

		if (recurrenceInfo) {
			var recurrenceCount = recurrenceInfo.countRecurrenceItems();
			for (var i = 0; i < recurrenceCount; i++) {
				var recurrenceItem = recurrenceInfo.getRecurrenceItemAt(i);
				if (recurrenceItem.isNegative && recurrenceItem instanceof Ci.calIRecurrenceDate) {
					exceptionIDs.push(recurrenceItem.date);
				} else {
					if (!recurrenceItem.isNegative && recurrenceItem instanceof Ci.calIRecurrenceRule) {

						// !isFinite ==> repeat forever
						// -> transform to "until event start date + 1 year"
						if (!recurrenceItem.isFinite) {
							endDate = aCalEvent.startDate.jsDate;
							endDate.setHours(23);
							endDate.setMinutes(59);
							endDate.setSeconds(59);
							endDate.setMilliseconds(999);
							endDate.setFullYear(endDate.getFullYear() + 1); // add a year
							// correct timezone offset
							endDate = new Date(endDate.getTime() - endDate.getTimezoneOffset() * 60 * 1000);
						} else {
							if (recurrenceItem.isByCount) {
								// end date ==> date of last occurrence
								var previousOccurence;
								var nextOccurrence = aCalEvent;

								do {
									previousOccurence = nextOccurrence;
									var occurrenceTime = previousOccurence.startDate;
									nextOccurrence = recurrenceInfo.getNextOccurrence(occurrenceTime);
								} while ( nextOccurrence != null );

								try {
									endDate = previousOccurence.endDate.jsDate;
								} catch (e) {
									endDate = previousOccurence.untilDate.jsDate;
								}
							} else { // recurrence lasts up to a certain date
								try {
									endDate = recurrenceItem.endDate.jsDate;
								} catch (e) {
									endDate = recurrenceItem.untilDate.jsDate;
								}
							}
						}

						switch (recurrenceItem.type) {
						case "DAILY":

							var onDays = recurrenceItem.getComponent("BYDAY", {});
							if (onDays.length > 0) {
								//on every weekday
								var days = ['0', '0', '0', '0', '0', '0', '0'];
								for (var j = 0; j < onDays.length; j++) {
									days[onDays[j] - 1] = '1';
								}
								days = days.join("");
								// make Sunday day 7
								days = days.replace(/(0|1)([0|1]{6})/, "$2$1");

								aObmEvent.kind = "weekly";
								aObmEvent.frequence = recurrenceItem.interval;
								aObmEvent.days = days;
								break;
							}
							aObmEvent.kind = "daily";
							aObmEvent.frequence = recurrenceItem.interval;
							aObmEvent.days = "";
							break;

						case "WEEKLY":
							aObmEvent.kind = "weekly";
							aObmEvent.frequence = recurrenceItem.interval;
							var onDays = recurrenceItem.getComponent("BYDAY", {});
							var days = ['0', '0', '0', '0', '0', '0', '0'];
							if (!onDays || onDays === null || onDays.lengh === 0 || onDays == "") {
								// lightning bug : no day selected
								// -> no day stored but start day of event used in views
								days[aCalEvent.startDate.weekday] = '1';
							} else {
								for (var j = 0; j < onDays.length; j++) {
									days[onDays[j] - 1] = '1';
								}
							}
							days = days.join("");
							// make Sunday day 7
							days = days.replace(/(0|1)([0|1]{6})/, "$2$1");
							aObmEvent.days = days;
							break;

						case "MONTHLY":
							//dump("recc monthly: \n");
							var onDays = recurrenceItem.getComponent("BYDAY", {});
							//dump("recc, onDays" + onDays[0] + "\n");
							if (onDays.length == 1) {
								aObmEvent.kind = "monthlybyday";
								aObmEvent.frequence = recurrenceItem.interval;
								aObmEvent.days = "";
								break;
							}
							onDays = recurrenceItem.getComponent("BYMONTHDAY", {});
							//dump("recc, onDays" + onDays[0] + "\n");
							if (onDays.length == 1) {
								aObmEvent.kind = "monthlybydate";
								aObmEvent.frequence = recurrenceItem.interval;
								aObmEvent.days = "";
								break;
							}
							// lightning bug : no day or date selected
							// force recc by date
							aObmEvent.kind = "monthlybydate";
							aObmEvent.frequence = recurrenceItem.interval;
							aObmEvent.days = "";
							break;
							//invalidChoice = true;
							//break;
						case "YEARLY":
							aObmEvent.kind = "yearly";
							aObmEvent.frequence = recurrenceItem.interval;
							aObmEvent.days = "";
							break;

							break;
						default:
							invalidChoice = true;
							break;
						}
					}
				}
			}
		}
		aObmEvent.exceptions = [];
		aObmEvent.getExceptions = function(count) {
			count.value = this.exceptions.length;
			return this.exceptions;
		};

		if (!recurrenceInfo || invalidChoice) {
			aObmEvent.kind = "none";
			aObmEvent.frequence = 1;
			aObmEvent.days = "";
		} else {
			exceptionIDs = exceptionIDs.concat(recurrenceInfo.getExceptionIds({}));
			for each(var exceptionID in exceptionIDs) {
				aObmEvent.exceptions.push(UtilsOBM.jsDateToXMLDate(exceptionID.jsDate).toString());
			}
		}
		if (recurrenceItem && !recurrenceItem.isFinite) {
			aObmEvent.end = "2038-01-19T06:14:07.000Z";
		} else {
			aObmEvent.end = UtilsOBM.jsDateToXMLDate(endDate);
		}

		// OBM can't handle main event if it is not in recurrence (Bug #171)
		// So event is set to be on the first occurrence
		if (recurrenceInfo) {
			var aDate = aCalEvent.startDate.clone();
			aDate.resetTo(aDate.year, aDate.month, aDate.day, aDate.hour, aDate.minute, aDate.second - 1, aDate.timezone);
			var firstRecurrenceItem = recurrenceInfo.getNextOccurrence(aDate);
			if (firstRecurrenceItem && aCalEvent.startDate.compare(firstRecurrenceItem.startDate)) {
				this.logger.debug("First occurrence of the event (" + firstRecurrenceItem.startDate + ") is not the start date (" + aCalEvent.startDate + "). Start date is set to first occurence.", false, false);
				aObmEvent.date = UtilsOBM.jsDateToRestDate(firstRecurrenceItem.startDate.jsDate);
			}
		}
	},

	//
	// Resolve conflicts following override rule
	//
	_resolveConflicts: function(calName) {
		this.logger.debug("_resolveConflicts()", false, false);
		var serverPriority = (UtilsOBM.getCharPref("extensions.obm.override", "server") == "server");
		var remoteListHasChanged = false;
		for (var i = 0; i < this._remoteDiff[calName].removed.length; i++) {
			var eventId = this._remoteDiff[calName].removed[i];
			var rLocalIdx = this._localDiff[calName].event.removed.indexOf(eventId);
			var uLocalIdx = this._localDiff[calName].event.updated.indexOf(eventId);
			if (rLocalIdx != -1) {
				// A conflict
				this._remoteDiff[calName].removed.splice(i--, 1);
				this._localDiff[calName].event.removed.splice(rLocalIdx, 1);
				UtilsOBM.removeItemFromChangesList(Ci.calICalendar.toString(), calName, Ci.calIEvent.toString(), "removed", eventId);
				remoteListHasChanged = true;
			}
			if (uLocalIdx != -1) {
				// A conflict
				if (serverPriority) {
					this._localDiff[calName].event.updated.splice(uLocalIdx, 1);
					UtilsOBM.removeItemFromChangesList(Ci.calICalendar.toString(), calName, Ci.calIEvent.toString(), "updated", eventId);
				} else {
					if (!remoteListHasChanged) this._remoteDiff[calName].removed.splice(i--, 1);
				}
			}
		}
		for (var i = 0; i < this._remoteDiff[calName].updated.length; i++) {
			remoteListHasChanged = false;
			var eventId = this._remoteDiff[calName].updated[i].uid;
			var rLocalIdx = this._localDiff[calName].event.removed.indexOf(eventId);
			var uLocalIdx = this._localDiff[calName].event.updated.indexOf(eventId);
			if (uLocalIdx != -1) {
				// A conflict
				if (serverPriority) {
					this._localDiff[calName].event.updated.splice(uLocalIdx, 1);
					UtilsOBM.removeItemFromChangesList(Ci.calICalendar.toString(), calName, Ci.calIEvent.toString(), "updated", eventId);
				} else {
					this._remoteDiff[calName].updated.splice(i--, 1);
					remoteListHasChanged = true;
				}
			}
			if (rLocalIdx != -1) {
				// A conflict
				if (serverPriority) {
					this._localDiff[calName].event.removed.splice(uLocalIdx, 1);
					UtilsOBM.removeItemFromChangesList(Ci.calICalendar, calName, Ci.calIEvent.toString(), "removed", eventId);
				} else {
					if (!remoteListHasChanged) this._remoteDiff[calName].updated.splice(i--, 1);
				}
			}
		}

	},

	_getSubscribedOBMCalendar: function() {
		this.logger.debug("_getSubscribedOBMCalendar()", false, false);
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		var calendars = calMgr.getCalendars({});
		var res = new Array();
		for each(var calendar in calendars) {
			if (calendar.type == "obm") {
				res.push(calendar);
			}
		}
		return res;
	},
	_getLocalDiff: function() {
		this.logger.debug("_getLocalDiff()", false, false);
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		this.callerListener.onNewOperation("getLocalDIff", null);
		if (this._justOneCalendar) this.calendars = [this._selectedCalendar];
		else this.calendars = this._getSubscribedOBMCalendar();

		for (var i = 0; i < this.calendars.length; i++) {
			var calName = calMgr.getCalendarPref_(this.calendars[i], "X-OBM-NAME");
			var changeTypes = ["added", "updated", "removed"];
			this._localDiff[calName] = {};
			this._localDiff[calName]["event"] = {};
			this._localDiff[calName]["task"] = {};

			for each(var type in changeTypes) {
				var pref = UtilsOBM.getCharPref("extensions.obm.changes." + Ci.calICalendar + "." + calName + "." + Ci.calIEvent + "." + type, "");
				this._localDiff[calName].event[type] = (pref != "" ? pref.split(",") : []);

				pref = UtilsOBM.getCharPref("extensions.obm.changes." + Ci.calICalendar + "." + calName + "." + Ci.calITodo + "." + type, "");
				this._localDiff[calName].task[type] = (pref != "" ? pref.split(",") : []);
			}

			var checkTypes = ["added", "updated"];
			for each(var type in checkTypes) {
				for each(var uid in this._localDiff[calName].event[type]) {
					var addedIds = this._exceptionToItem(this.calendars[i], uid);
					for each(var id in addedIds) {
						this._localDiff[calName].event["added"].push(id);
					}
				}
			}
		}

		this.callerListener.onOperationDone(true, null);

		this.callerListener.onNewOperation("getSync", null);
		this._getSyncOperationCount = this.calendars.length;
		var self = this;
		this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
			self._onGetSync(aResCode, aMethodName, aResult, aData)
		});

		this._localOperations = [];
		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
			this._localOperations[i] = [cal, calName, i, this.calendars.length,
			function(aCal, aCalName, aIdx, aTotal) {
				self.callerListener.onProgress(aIdx, aTotal, "", aCalName);
				var lastSync = self._getLastSync(aCal);
				var data = new Context();
				data.methodName = "getSync";
				data.isTheFirstPass = true;
				data.calendar = aCal;
				data.calName = aCalName;
				data.lastSync = lastSync;
				self.CalProxy.getSync(aCalName, lastSync, data);
			}];
		}

		this._localOperationsIdx = 0;
		this._sendNextLocalOperation();
	},
	_sendNextLocalOperation: function() {
		var idx = this._localOperationsIdx++;
		this._localOperations[idx][4](this._localOperations[idx][0], this._localOperations[idx][1], this._localOperations[idx][2], this._localOperations[idx][3]);
	},
	_exceptionToItem: function(calendar, aId) {

		this.logger.debug("_exceptionToItem()", false, false);
		var listener = {};
		this.addedIds = [];
		var self = this;
		listener.calendar = calendar;
		listener.onOperationComplete = function(aCalendar, aStatus, aOperationType, aId, aDetail) {
			// <TO_REMOVE>
			// Fix second onOperationComplete call (bug LT 0.8)
			// Fixed in LT 0.9
			try {
				if (aDetail.toString() == "Can't deduce item type based on QI") {
					return;
				}
			} catch(e) {}
			// </TO_REMOVE>
			if (Components.isSuccessCode(aStatus)) {
				var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
				var calName = calMgr.getCalendarPref_(this.calendar, "X-OBM-NAME");
				var localEvent = this.localEvent;

				//get exception
				try {
					if (localEvent.recurrenceInfo != null) {
						var aExceptionIDs = localEvent.recurrenceInfo.getExceptionIds({});
						// has Exception
						if (aExceptionIDs.length != 0) {

							for each(var aExceptionID in aExceptionIDs) {
								var ocurrence = localEvent.recurrenceInfo.getOccurrenceFor(aExceptionID, false);

								//var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
								var calEvent = Cc["@mozilla.org/calendar/event;1"].createInstance(Ci.calIEvent);
								var uuidgen = Components.classes["@mozilla.org/uuid-generator;1"].getService(Components.interfaces.nsIUUIDGenerator);
								var uuid = uuidgen.generateUUID().toString();

								calEvent.calendar = this.calendar;
								calEvent.id = uuid.substring(1, uuid.length - 1);
								calEvent.title = ocurrence.title;
								calEvent.setProperty("DESCRIPTION", ocurrence.getProperty("DESCRIPTION"));
								calEvent.setProperty("LOCATION", ocurrence.getProperty("LOCATION"));
								calEvent.startDate = ocurrence.startDate;
								calEvent.endDate = ocurrence.endDate;
								calEvent.setProperty("CATEGORIES", ocurrence.getProperty("CATEGORIES"));
								calEvent.priority = ocurrence.priority;
								calEvent.privacy = ocurrence.privacy;

								var attendees = ocurrence.getAttendees({});
								for each(var attendee in attendees) {
									calEvent.addAttendee(attendee);
								}

								var listenerAdd = {};
								var caller = this;
								listenerAdd.pcaller = self;
								listenerAdd.onOperationComplete = function(cal, res, op, id, item) {
									this.pcaller.addedIds.push(id);
								};
								this.calendar.adoptItem(calEvent, listenerAdd);
							} //end for
						}
					}
				} catch(e) {
					self.logger.debug("error getting exception items: " + e, false, false);
				}
			}
		};
		listener.onGetResult = function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
			if (aCount > 0) {
				this.localEvent = aItems[0];
			}
		};

		calendar.getItem(aId, listener);
		return this.addedIds;
	},
	_onReLogged: function(aResCode, aMethodName, aResult, aData) {
		this.logger.debug("_onReLogged()", false, false);
		this.CalProxy.removeListener();
		if (!this._ensureValidResponse(aResCode, aMethodName, aResult, "logUserIn")) {
			return;
		}

		var data = aData.wrappedJSObject;
		var self = this;
		// recall method with params in aData
		switch (data.methodName) {
		case "getSync":
			this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
				self._onGetSync(aResCode, aMethodName, aResult, aData)
			});
			this.CalProxy.getSync(data.calName, data.lastSync, aData);
			break;
		case "createEvent":
			this.logger.debug("retry createEvent()", false, false);
			this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
				self._onSentLocalDiff(aResCode, aMethodName, aResult, aData)
			});
			this.CalProxy.createEvent(data.calName, data.eventToSend, aData);
			break;
		case "modifyEvent":
			this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
				self._onSentLocalDiff(aResCode, aMethodName, aResult, aData)
			});
			this.CalProxy.modifyEvent(data.calName, data.eventToSend, true, aData);
			break;
		case "removeEvent":
			this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
				self._onSentLocalDiff(aResCode, aMethodName, aResult, aData)
			});
			this.CalProxy.removeEvent(data.calName, data.id, aData);
			break;
		default:
			//
		}
	},
	_ensureValidResponse: function(aResCode, aMethodName, aResult, aValidMethodName, aData) {
		this.logger.debug("_ensureValidResponse(" + aMethodName + ")", false, false);
		if (aResCode != Cr.NS_OK) {
			var msgError = "unknown";
			if (aResult instanceof Ci.nsISupportsString) msgError = aResult;

			if (msgError == "Invalid access token") {
				this.logger.debug("Invalid access token -> re-login and retry", false, false);
				// try to re-login if ok re-call method
				var login = {};
				var password = {};
				this._getOBMCredentials(login, password);

				var self = this;
				this.CalProxy.removeListener();
				this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
					self._onReLogged(aResCode, aMethodName, aResult, aData);
				});

				this.CalProxy.logUserIn(login.value, password.value, aData);

			} else {
				var printError = this._printableError(aMethodName, msgError);
				msgError = aMethodName + ": Reported error: " + msgError + "(" + aResCode + ")";
				this.logger.error(msgError, true, false);
				if (this.callerListener) {
					if (this._isCriticalMethod(aMethodName)) {
						this.callerListener.onStop(aResCode, printError);
					} else {
						this.callerListener.onError(printError);
					}
				}
			}
			return false;
		}

		if (aValidMethodName && aMethodName != aValidMethodName) {
			var msgError = "Method " + aMethodName + " unexpected. Expected: " + aValidMethodName;
			this.logger.error(msgError, true, false);
			this.callerListener.onError(msgError);
			return false;
		}
		return true;
	},
	_onLogged: function(aResCode, aMethodName, aResult, aData) {
		this.logger.debug("_onLogged()", false, false);
		var self = this;
		this.CalProxy.removeListener();
		if (!this._ensureValidResponse(aResCode, aMethodName, aResult, "logUserIn")) {
			return;
		};
		this.callerListener.onOperationDone(true, null);

		if (this._OBMCalendarsArePresent() && !this._onlyListCalendars) {
			this._getLocalDiff();
		} else {
			this.callerListener.onNewOperation("listCalendars", null);
			var self = this;
			this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
				self._onListCalendar(aResCode, aMethodName, aResult, aData)
			});
			this.CalProxy.listCalendars(null);
		}
	},

	_onGetSync: function(aResCode, aMethodName, aResult, aData) {
		this.logger.debug("_onGetSync()", false, false);
		if (!this._ensureValidResponse(aResCode, aMethodName, aResult, "getSync", aData)) {
			return;
		};

		var data = aData.wrappedJSObject;
		var calendar = data.calendar;
		var calName = data.calName;
		var syncResponse = aResult.QueryInterface(Ci.obmICalendarSyncResponse);
		this._remoteDiff[calName] = {};
		this._remoteDiff[calName]["updated"] = syncResponse.getUpdatedEvents({});
		var lastSync = this._getLastSync(calendar);
		if (lastSync != "1970-01-01T00:00:00Z") {
			this._remoteDiff[calName]["removed"] = syncResponse.getRemovedEventIds({});
		} else {
			this._remoteDiff[calName]["removed"] = [];
		}
		if (data.isTheFirstPass) this._resolveConflicts(calName);
		this._getSyncOperationCount--;
		//this._setLastSync(calName, syncResponse.lastsync);
		this._remoteDiff[calName].lastSync = syncResponse.lastsync;

		if (this._getSyncOperationCount <= 0) {
			var self = this;
			this.CalProxy.removeListener();
			this.callerListener.onOperationDone(true, null);
			this._mergeRemoteDiff(data.isTheFirstPass);
		} else {
			this._sendNextLocalOperation();
		}
	},

	/**
 * Traduit une récurrence du format OBM au format Calendar.
 * 
 * @param {obmIEventRecurrence}
 *            aRecurrence
 * @pram {calIEvent} aCalEvent L'évènement auquel la récurrence est associée
 * @type calIRecurrence
 */
	_obmRecurrenceToCalRecurrence: function(aRecurrence, aCalEvent) {
		var recurrenceInfo = Cc["@mozilla.org/calendar/recurrence-info;1"].createInstance(Ci.calIRecurrenceInfo);
		recurrenceInfo.item = aCalEvent;

		var recRule = Cc["@mozilla.org/calendar/recurrence-rule;1"].createInstance(Ci.calIRecurrenceRule);

		switch (aRecurrence.kind) {
		case "daily":
			recRule.type = "DAILY";
			recRule.interval = aRecurrence.frequence;
			break;

		case "weekly":
			recRule.type = "WEEKLY";
			recRule.interval = aRecurrence.frequence;

			var onDays = [];

			var onDaysOBM = aRecurrence.days.split("");
			onDaysOBM.unshift(onDaysOBM.pop());

			for (var i = 0; i < onDaysOBM.length; i++) {
				if (onDaysOBM[i] == "1") {
					onDays.push((i + 1).toString());
				}
			}

			if (onDays.length > 0) {
				recRule.setComponent("BYDAY", onDays.length, onDays);
			}
			break;

		case "monthlybyday":
			recRule.type = "MONTHLY";
			recRule.interval = aRecurrence.frequence;
			recRule.setComponent("BYDAY", 1, [(Math.round(aCalEvent.startDate.day / 8) + 1) * 8 + aCalEvent.startDate.weekday + 1]);
			break;

		case "monthlybydate":
			recRule.type = "MONTHLY";
			recRule.interval = aRecurrence.frequence;
			recRule.setComponent("BYMONTHDAY", 1, [aCalEvent.startDate.day]);
			break;

		case "yearly":
			recRule.type = "YEARLY";
			recRule.interval = aRecurrence.frequence;
			break;

		case "none":
		default:
			return null;
			break;
		}

		if (!aRecurrence.end || aRecurrence.end == "") {
			recurrenceInfo.isFinite = false;
		} else {
			var endDate = jsDateToDateTime(UtilsOBM.xmlDateToJSDate(aRecurrence.end)).getInTimezone(this._timezone);
			endDate.timezone = this._timezone;
			if (!aCalEvent.startDate.isDate) {
				endDate.hour = aCalEvent.startDate.hour;
				endDate.minute = aCalEvent.startDate.minute;
				endDate.second = aCalEvent.startDate.second;
			} else {
				endDate.isDate = true;
			}
			dump("endDate.day:" + endDate + "\n");
			try {
				recRule.endDate = endDate;
			} catch (e) {
				recRule.untilDate = endDate;
			}
		}

		recurrenceInfo.insertRecurrenceItemAt(recRule, 0);

		// first, remove all current exception IDs
		var exceptionIDs = recurrenceInfo.getExceptionIds({});
		for each(var exceptionID in exceptionIDs) {
			recurrenceInfo.removeExceptionFor(exceptionID);
		}

		// now, add all "new" exception IDs
		var exceptions = aRecurrence.getExceptions({});
		for each(var exception in exceptions) {
			var exceptionID = jsDateToDateTime(UtilsOBM.xmlDateToJSDate(exception)).getInTimezone(this._timezone);
			exceptionID.timezone = this._timezone;

			if (aCalEvent.startDate.isDate) { // event is all day
				exceptionID.isDate = true;
			}

			try {
				recurrenceInfo.removeOccurrenceAt(exceptionID); // getExceptionFor(exceptionID,
				// true /* aCreate */);
			} catch(e) {
				// in case exception date isn't a valid occurence date of the recurrence
				// date set
			}
		}

		return recurrenceInfo;
	},

	_createLocalEvent: function(aEvent
	/* obmIEvent */
	, calendar) {
		this.logger.debug("_createLocalEvent()", false, false);
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		var calEvent = Cc["@mozilla.org/calendar/event;1"].createInstance(Ci.calIEvent);
		calEvent.calendar = calendar;
		dump("calendar.name:" + calendar.name + "\n");
		calEvent.id = aEvent.uid;
		calEvent.title = aEvent.title;
		if (aEvent.description != "") {
			calEvent.setProperty("DESCRIPTION", aEvent.description);
		}
		if (aEvent.location != "") {
			calEvent.setProperty("LOCATION", aEvent.location);
		}
		var jsStartDateTime = jsDateToDateTime(UtilsOBM.xmlDateToJSDate(aEvent.date)).getInTimezone(this._timezone);
		jsStartDateTime.timezone = this._timezone;
		calEvent.startDate = jsStartDateTime;

		var seconds;

		if (aEvent.allday) {
			calEvent.startDate.isDate = true;
			// round duration to the day
			seconds = (Math.floor(aEvent.duration / 86400) + 1) * 86400;
		} else {
			calEvent.startDate.isDate = false;
			seconds = aEvent.duration;
		}

		var jsEndDateTime = jsStartDateTime.clone();
		var duration = Cc["@mozilla.org/calendar/duration;1"].createInstance(Ci.calIDuration);
		duration.inSeconds = seconds;
		jsEndDateTime.addDuration(duration);
		calEvent.endDate = jsEndDateTime;
		calEvent.endDate.isDate = aEvent.allday;

		if (aEvent.category && aEvent.category != "") {
			if (calEvent.setCategories) {
				// lightning 0.9 support
				calEvent.setCategories(1, ["[OBM] " + aEvent.category]);
			} else {
				calEvent.setProperty("CATEGORIES", "[OBM] " + aEvent.category);
			}
		}

		switch (aEvent.priority) {
		case 1:
			calEvent.priority = 9;
			break;
		case 2:
			calEvent.priority = 5;
			break;
		case 3:
			calEvent.priority = 1;
			break;
		default:
			calEvent.priority = 5;
		}

		calEvent.privacy = (aEvent.classification == 1 ? "PRIVATE": "PUBLIC");

		var attendees = aEvent.getAttendees({});
		for each(var attendeeElem in attendees) {
			var calAttendee = Cc["@mozilla.org/calendar/attendee;1"].createInstance(Ci.calIAttendee);
			calAttendee.id = "MAILTO:" + attendeeElem.email;
			calAttendee.commonName = attendeeElem.displayName;
			switch (attendeeElem.status) {
			case "ACCEPTED":
				calAttendee.participationStatus = "ACCEPTED";
				break;
			case "REFUSED":
				calAttendee.participationStatus = "DECLINED";
				break;
			case "WAITING":
				calAttendee.participationStatus = "NEEDS-ACTION";
				break;
			default:
				calAttendee.participationStatus = "NEEDS-ACTION";
			}
			switch (attendeeElem.type) {
			case "REQUIRED":
				calAttendee.role = "REQ-PARTICIPANT";
				break;
			default:
				calAttendee.role = "OPT-PARTICIPANT";
			}
			calEvent.addAttendee(calAttendee);
		}

		var ownerEmail = calMgr.getCalendarPref_(calendar, "X-OBM-EMAIL");

		var attendee = calEvent.getAttendeeById("MAILTO:" + ownerEmail);
		if (attendee) {
			switch (attendee.participationStatus) {
			case "ACCEPTED":
				calEvent.status = "CONFIRMED";
				break;
			case "DECLINED":
				calEvent.status = "CANCELLED";
				break;
			case "NEEDS-ACTION":
				calEvent.status = "TENTATIVE";
				break;
			default:
				calEvent.status = "TENTATIVE";
			}
		}

		var calRecurrence = this._obmRecurrenceToCalRecurrence(aEvent.recurrence, calEvent);
		if (calRecurrence) {
			calEvent.recurrenceInfo = calRecurrence;
		}

		if (this._supportAlert) {
			if (calEvent.alarmOffset === undefined) {
				calEvent.clearAlarms();
			}
			if (aEvent.alert != null && aEvent.alert != -1) {
				var alert = Cc["@mozilla.org/calendar/duration;1"].createInstance(Ci.calIDuration);
				alert.inSeconds = -1 * aEvent.alert; // Alarm before event starts -> negative duration | OBM manage positive durations
				if (calEvent.alarmOffset === undefined) {
					var alarm = createAlarm();
					alarm.related = Ci.calIAlarm.ALARM_RELATED_START;
					alarm.offset = alert;
					calEvent.addAlarm(alarm);
				} else {
					calEvent.alarmOffset = alert;
				}
				if (calEvent.recurrenceInfo) {
					calEvent.alarmLastAck = now();
				}
			}
		}

		return calEvent;
	},
	_updateOrAdoptEvent: function(calendar, event, isTheFirstPass, notify) {
		this.logger.debug("_updateOrAdoptEvent()", false, false);
		var listener = {};
		var self = this;
		if (notify) listener.isTheFirstPass = isTheFirstPass; // Closure should be enough
		listener.newItem = event;
		listener.calendar = calendar;
		listener.onOperationComplete = function(aCalendar, aStatus, aOperationType, aId, aDetail) {
			// <TO_REMOVE>
			// Fix second onOperationComplete call (bug LT 0.8)
			// Fixed in LT 0.9
			try {
				if (aDetail.toString() == "Can't deduce item type based on QI") {
					return;
				}
			} catch(e) {}
			// </TO_REMOVE>
			//dump("--aStatus: " + aStatus + " aOperationType: " + aOperationType + " aId: " + aId + " aDetail: " + aDetail + "\n");
			// FIXME : bug lightning ? aStatus for getItem is allways 0 
			if (!Components.isSuccessCode(aStatus) || !this.localEvent) {
				if (this.newItem.status != "CANCELLED") {
					self.logger.debug("Adopt item : " + this.newItem.title, false, false);
					this.calendar.adoptItem(this.newItem, null);
				}
			} else {
				// Will work with LT 0.9 ... not !
				var localEvent;
				try {
					localEvent = aDetail.QueryInterface(Ci.calIEvent)
				} catch(e) {
					localEvent = this.localEvent;
				}
				if (this.newItem.status != "CANCELLED") {
					self.logger.debug("Update item !", false, false);
					this.calendar.modifyItem(this.newItem
					/* aNewItem */
					, localEvent
					/* aOldItem */
					, null
					/* listener */
					);
				} else {
					self.logger.debug("Remove item !", false, false);
					this.calendar.deleteItem(localEvent, null
					/* listener */
					);
				}
			}
			if (notify) self._aMergeActionCompleted(this.isTheFirstPass);
		};
		listener.onGetResult = function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
			if (aCount > 0) {
				this.localEvent = aItems[0];
			}
		};

		calendar.getItem(event.id, listener);
	},
	_removeEvent: function(calendar, eventId, isTheFirstPass, notify) {
		var listener = {};
		var self = this;
		listener.isTheFirstPass = isTheFirstPass; // Closure should be enough
		listener.calendar = calendar;
		listener.onOperationComplete = function(aCalendar, aStatus, aOperationType, aId, aDetail) {
			try {
				if (aDetail.toString() == "Can't deduce item type based on QI") {
					return;
				}
			} catch(e) {}

			if (aStatus == Cr.NS_OK) {
				// Found, so must be deleted
				self.logger.debug("Remove item !", false, false);
				if (this.localItem) {
					try {
						this.calendar.deleteItem(this.localItem, null
						/* listener */
						)
					} catch(e) {};
				}
			}
			if (notify) self._aMergeActionCompleted(this.isTheFirstPass);

		};
		listener.onGetResult = function(aCalendar, aStatus, aItemType, aDetail, aCount, aItems) {
			if (aCount > 0) {
				this.localItem = aItems[0];
			}
		};
		calendar.getItem(eventId, listener);
	},
	_aMergeActionCompleted: function(isTheFirstPass) {
		if (this.mergeActionCount > 0) {
			this.callerListener.onProgress(this.mergeActionIdx, this.mergeActionCount, "", "all");
			this.logger.debug("a action completed:" + this.mergeActionIdx, false, false);
		}
		this.mergeActionIdx++;
		/*if (this.mergeActionIdx >= this.mergeActionCount) {
		this.logger.debug("Merge completed", false, false);
		// Let's go !!!
		for (var i = 0; i < this.calendars.length; i++) {
		var cal = this.calendars[i];
		cal.endBatch();
		this.logger.debug("endBatch", false, false);
		}

		this.callerListener.onOperationDone(true, null);
		if (isTheFirstPass) {
		return (this._sendLocalDiff());
		} else {
		this.callerListener.onStop(Cr.NS_OK, "done");
		}
	} else {
		//return (this._sendNextLocalOperation());
	}*/
	},
	_isEventPrivateFor: function(event, calName) {
		//dump("calName: "+ calName + "\n");
		//dump("login: " + this._login + "\n");
		//dump("event.owner: "+ event.owner +"\n");
		//dump("event.classification: "+ event.classification +"\n");
		var attendees = event.getAttendees({});
		//dump("event.attendees: " + attendees + "\n");
		if (!event.classification == 1) {
			return false;
		}
		if (event.owner == this._login.split("@")[0]) {
			return false;
		}
		if (!attendees) {
			return true;
		}
		for each(attendee in attendees) {
			//dump("attendee.email: " + attendee.email + "\n");
			if (attendee.email == this._login.split("@")[0]) {
				return false;
				break;
			}
		}
		return true;
	},
	_mergeRemoteDiff: function(isTheFirstPass) {
		this.logger.debug("_mergeRemoteDiff()", false, false);
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);

		this.mergeActionCount = 0;
		this.mergeActionIdx = 0;
		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
			this.mergeActionCount += this._remoteDiff[calName].updated.length;
			this.mergeActionCount += this._remoteDiff[calName].removed.length;
		}

		this.callerListener.onNewOperation("mergeRemoteDiff", this.mergeActionCount);
		this.logger.debug("Action count: " + this.mergeActionCount, false, false);

		var self = this;

		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			this.logger.debug("startBatch", false, false);
			cal.startBatch();
			var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
			for each(var event in this._remoteDiff[calName].updated) {
				var calEvent = this._createLocalEvent(event, cal);
				if (this._isEventPrivateFor(event, calName) || calEvent.status == "CANCELLED") {
					this.logger.debug("event is private or refused (remove if exist)", false, false);
					this._removeEvent(cal, event.uid, isTheFirstPass, true);
				} else {
					this._updateOrAdoptEvent(cal, calEvent, isTheFirstPass, true);
				}
			}
			for each(var event in this._remoteDiff[calName].removed) {
				this._removeEvent(cal, event, isTheFirstPass, true);
			}
			cal.endBatch();
			this._setLastSync(calName, this._remoteDiff[calName].lastSync);
		}
		if (this.mergeActionCount == 0) {
			this.logger.debug("No remote event have been updated/added/removed", false, false);
			this._aMergeActionCompleted(isTheFirstPass);
		}
		this.logger.debug("Merge completed", false, false);
		this.callerListener.onOperationDone(true, null);
		if (isTheFirstPass) {
			this._sendLocalDiff();
		} else {
			//end sync
			this.callerListener.onStop(Cr.NS_OK, "done");
		}
		/*this._localOperations = [];
		for (var i = 0; i < this.calendars.length; i++) {
		var cal = this.calendars[i];
		this.logger.debug("startBatch", false, false);
		cal.startBatch();
		var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME")
		var idx = 0;
		for (var j=0; j < this._remoteDiff[calName].updated.length; j++) {
			this._localOperations[idx++] = [cal, j, this._remoteDiff[calName].updated.length,
				function(cal, aIdx, aTotal){
					var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
					var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
					var event = self._remoteDiff[calName].updated[aIdx];
					self.callerListener.onProgress(aIdx, aTotal, "updated", calName);
					if (self._isEventPrivateFor(event, calName)) {
						self.logger.debug("event is private (remove if exist)", false, false);
						return (self._removeEvent(cal, event.uid, isTheFirstPass, true));
					} else {
						var calEvent = self._createLocalEvent(event, cal);
						return (self._updateOrAdoptEvent(cal, calEvent, isTheFirstPass, true));
					}
				}
			];
		}
		for (var j=0; j < this._remoteDiff[calName].removed.length; j++) {
			this._localOperations[idx++] = [cal, j, this._remoteDiff[calName].removed.length,
				function(cal, aIdx, aTotal){
					var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
					var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
					var event = self._remoteDiff[calName].removed[aIdx];
					self.callerListener.onProgress(aIdx, aTotal, "removed", calName);
					return (self._removeEvent(cal, event, isTheFirstPass, true));
				}
			];
		}
		}
		this._localOperationsIdx = 0;
		return (this._sendNextLocalOperation());
	}*/
	},
	_onSentLocalDiff: function(aResCode, aMethodName, aResult, aData) {
		if (!this._ensureValidResponse(aResCode, aMethodName, aResult, aMethodName, aData)) {
			if ((aResult instanceof Ci.nsISupportsString) && (aResult == "Invalid access token")) {
				// do not continue -> must wait reLogin
				return;
			} else {
				// continue -> sync algo permit local operation to fail
				this._localDiffOperationCount--;
			}
		} else {
			this._localDiffOperationCount--;
			switch (aMethodName) {
			case "createEvent":
				var newId = this._getJSStringFromXPString(aResult);
				var ctx = aData.wrappedJSObject;
				var newLocalEvent = ctx.localEvent.clone();
				newLocalEvent.id = newId;
				ctx.calendar.modifyItem(newLocalEvent, ctx.localEvent, null);
				break;
			case "modifyEvent":
				var ctx = aData.wrappedJSObject;
				var event = aResult.QueryInterface(Ci.obmIEvent);
				var calEvent = this._createLocalEvent(event, ctx.calendar);
				this._updateOrAdoptEvent(ctx.calendar, calEvent, null, false);
				break;
			case "removeEvent":
				break;
			default:
			};
		}
		if (this._localDiffOperationCount <= 0) {
			this.callerListener.onOperationDone(true, null);
			for (var i = 0; i < this.calendars.length; i++) {
				var cal = this.calendars[i];
				cal.endBatch();
			}
			var self = this;
			this.CalProxy.removeListener();
			var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
			this.logger.debug("timer init", false, false);
			this.callerListener.onNewOperation("wait-one-seconde", null);
			timer.initWithCallback({
				notify: function(timer) {
					self.logger.debug("ok, time is out", false, false);
					return (self._fireSecondPass());
				}
			},
			1100, timer.TYPE_ONE_SHOT);
			this.timer = timer;
		} else {
			this._sendNextLocalOperation();
		}
	},
	_fireSecondPass: function() {
		this.logger.debug("_fireSecondPass()", false, false);

		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			cal.endBatch();
		}

		this._getSyncOperationCount = this.calendars.length;
		var self = this;
		this.callerListener.onOperationDone(true, null);
		this.callerListener.onNewOperation("getSync", null);
		this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
			self._onGetSync(aResCode, aMethodName, aResult, aData)
		});

		this._localOperations = [];
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
			this._localOperations[i] = [cal, calName, i, this.calendars.length,
			function(aCal, aCalName, aIdx, aTotal) {
				self.callerListener.onProgress(aIdx, aTotal, "", aCalName);
				var lastSync = self._getLastSync(aCal);
				var data = new Context();
				data.methodName = "getSync";
				data.isTheFirstPass = false;
				data.calendar = aCal;
				data.lastSync = lastSync;
				data.calName = aCalName;
				self.CalProxy.getSync(aCalName, lastSync, data);
			}];
		}

		this._localOperationsIdx = 0;
		return (this._sendNextLocalOperation());
	},
	_sendLocalDiff: function() {
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			this.logger.debug("startBatch", false, false);
			cal.startBatch();
		}

		this._localDiffOperationCount = 0;
		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
			this._localDiffOperationCount += this._localDiff[calName].event.removed.length;
			this._localDiffOperationCount += this._localDiff[calName].event.updated.length;
			this._localDiffOperationCount += this._localDiff[calName].event.added.length;
		}

		this.logger.debug("_sendLocalDiff(" + this._localDiffOperationCount + ")", false, false);
		this.callerListener.onNewOperation("sendLocalDiff", this._localDiffOperationCount);

		var self = this;
		this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
			self._onSentLocalDiff(aResCode, aMethodName, aResult, aData)
		});

		if (this._localDiffOperationCount == 0) {
			return (this._onSentLocalDiff(Cr.NS_OK, null, null, null));
		}

		var idx = 0;
		this._localOperations = [];
		for (var i = 0; i < this.calendars.length; i++) {
			var cal = this.calendars[i];
			var calName = calMgr.getCalendarPref_(cal, "X-OBM-NAME");
			for (var j = 0; j < this._localDiff[calName].event.removed.length; j++) {
				this._localOperations[idx++] = [cal, calName, j, this._localDiff[calName].event.removed.length,
				function(aCal, aCalName, aIdx, aTotal) {
					self.callerListener.onProgress(aIdx, aTotal, "removed", aCalName);
					var id = self._localDiff[aCalName].event.removed[aIdx];
					var data = new Context();
					data.methodName = "removeEvent";
					data.calName = aCalName;
					data.id = id;
					self.CalProxy.removeEvent(aCalName, id, data);
					UtilsOBM.removeItemFromChangesList(Ci.calICalendar, aCalName, Ci.calIEvent, "removed", id);
				}];
			}
			for (var j = 0; j < this._localDiff[calName].event.added.length; j++) {
				this._localOperations[idx++] = [cal, calName, j, this._localDiff[calName].event.added.length,
				function(aCal, aCalName, aIdx, aTotal) {
					self.callerListener.onProgress(aIdx, aTotal, "added", aCalName);
					var id = self._localDiff[aCalName].event.added[aIdx];
					self._createEventObjFromId(aCal, id, true);
				}];
			}
			for (var j = 0; j < this._localDiff[calName].event.updated.length; j++) {
				this._localOperations[idx++] = [cal, calName, j, this._localDiff[calName].event.updated.length,
				function(aCal, aCalName, aIdx, aTotal) {
					self.callerListener.onProgress(aIdx, aTotal, "updated", aCalName);
					var id = self._localDiff[aCalName].event.updated[aIdx];
					self._createEventObjFromId(aCal, id, false);
				}];
			}
		}

		this._localOperationsIdx = 0;
		this._sendNextLocalOperation();
	},
	_OBMCalendarsArePresent: function() {
		this.logger.debug("_ensureOBMCalendarsArePresent()", false, false);
		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		var calendars = calMgr.getCalendars({});
		for each(var calendar in calendars) {
			if (calendar.type == "obm") return true;
		}
		return false;
	},

	// /////////////////////////////
	/*
	 * obmICalendarSync
	 */

	_logUserIn: function() {
		this.logger.debug("_logUserIn()", false, false);
		this.callerListener.onNewOperation("logUserIn", null);
		var login = {};
		var password = {};
		this._getOBMCredentials(login, password);
		this._login = login.value;

		var self = this;

		this.CalProxy.addListener(function(aResCode, aMethodName, aResult, aData) {
			self._onLogged(aResCode, aMethodName, aResult, aData)
		});
		this.CalProxy.logUserIn(login.value, password.value, null);
	},

	_subscribeTo: function(aCalInfo) {
		this.logger.debug("_subscribeTo()", false, false);
		this.callerListener.onNewOperation("subscribingToCal", null);

		var uid = aCalInfo.uid;
		var name = UtilsOBM.getCharPref("extensions.obm.autoconf.calname", null);
		if (!name) {
			name = "[OBM] " + aCalInfo.firstname + " " + aCalInfo.lastname;
		}
		var email = aCalInfo.mail;
		var color = UtilsOBM.getRandomColor();
		var readonly = !aCalInfo.write;
		this.logger.debug("_subscribeTo: " + uid + ", " + name + ", " + email + ", " + color + ", " + readonly, false, false);

		var calMgr = Cc["@mozilla.org/calendar/manager;1"].getService(Ci.calICalendarManager);
		UtilsOBM.setBoolPref("extensions.obm.changes.calICalendar." + uid + ".readOnly", readonly);

		var uri = Cc["@mozilla.org/network/simple-uri;1"].createInstance(Ci.nsIURI);
		uri.spec = "moz-profile-calendar://?id=" + UtilsOBM.getNextLocalCalendarID() + ";" + uid;
		this.logger.debug("uri.spec: " + uri.spec, false, false);
		var newCalendar = calMgr.createCalendar("obm", uri);
		calMgr.registerCalendar(newCalendar);
		newCalendar.name = name;
		calMgr.setCalendarPref_(newCalendar, "X-OBM-NAME", uid.toString());
		calMgr.setCalendarPref_(newCalendar, 'X-OBM-EMAIL', email.toString());
		newCalendar.setProperty("color", color.toString());
		this._setLastSync(uid, "1970-01-01T00:00:00Z");

		this.callerListener.onOperationDone(true, null);
		return newCalendar;
	},

	_onListCalendar: function(aResCode, aMethodName, aResult, aData) {
		this.logger.debug("_onListCalendar()", false, false);
		this.callerListener.onOperationDone(true, null);
		if (!this._ensureValidResponse(aResCode, aMethodName, aResult, "listCalendars")) {
			return;
		};
		var self = this;
		this.CalProxy.removeListener();

		if (this._onlyListCalendars) {
			this.callerListener.onOperationDone(true, aResult);
			this.callerListener.onStop(Cr.NS_OK, "done");
			return;
		}

		var calendars = aResult.QueryInterface(Ci.nsIArray);
		var count = calendars.length;
		var login = {};
		this._getOBMCredentials(login, {});
		login = login.value.split('@')[0];
		for (var i = 0; i < count; i++) {
			var calInfo = calendars.queryElementAt(i, Ci.obmICalendarInfo);
			if (calInfo.uid == login) {
				this._subscribeTo(calInfo);
				break;
			}
		}
		this._getLocalDiff();
	},

	mergeOneCalendar: function(aCalendar, aListener) {
		this.logger.debug("mergeOneCalendar()", false, false);
		this._justOneCalendar = true;
		this._selectedCalendar = aCalendar;
		this.merge(aListener);
	},

	mergeAllCalendars: function(aListener) {
		this.logger.debug("mergeAllCalendars()", false, false);
		this._justOneCalendar = false;
		this.merge(aListener);
	},

	getSubscribleCalendars: function(aListener) {
		this.logger.debug("getSubscribleCalendars()", false, false);
		this.callerListener = aListener;
		this.callerListener.onStart();
		this._onlyListCalendars = true;

		try {
			var WSDLUrl = UtilsOBM.getCharPref("extensions.obm.server") + "/Calendar";
		} catch(e) {
			var error = "NO_SERVER_INFO";
			this.logger.error(error, true, false);
			this.callerListener.onError(error);
			throw (error);
		}

		this.CalProxy = Cc["@obm.org/obm-sync/calendarproxy;1"].createInstance(Ci.obmICalendarProxy);
		this.CalProxy.WSDLUrl = WSDLUrl;

		if (UtilsOBM.getBoolPref("extensions.obm.calEvents.is_sync_enabled", true) == false) {
			this.callerListener.onStop(Cr.NS_OK, "done - Events not sync-able");
			return;
		}

		this._logUserIn();
		return;
	},

	merge: function(aListener) {
		this.logger.debug("mergeCalendars", false, false);
		this.callerListener = aListener;
		this.callerListener.onStart();

		try {
			var WSDLUrl = UtilsOBM.getCharPref("extensions.obm.server") + "/Calendar";
		} catch(e) {
			var error = "NO_SERVER_INFO";
			this.logger.error(error, true, false);
			this.callerListener.onError(error);
			throw (error);
		}

		this.CalProxy = Cc["@obm.org/obm-sync/calendarproxy;1"].createInstance(Ci.obmICalendarProxy);
		this.CalProxy.WSDLUrl = WSDLUrl;

		if (UtilsOBM.getBoolPref("extensions.obm.calEvents.is_sync_enabled", true) == false) {
			this.callerListener.onStop(Cr.NS_OK, "done - Events not sync-able");
			return;
		}

		this._logUserIn();
		return;
	}
};

// module
var ObmCalendarSyncModule = {
	mCID: Components.ID("{61907d2d-340e-4c17-9361-d43c155a6d5f}"),
	mContractID: "@obm.org/obm-sync/calendarsync;1",
	registerSelf: function(compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.mCID, "OBM Calendar Sync", this.mContractID, fileSpec, location, type);
	},
	getClassObject: function(compMgr, cid, iid) {

		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}
		if (!this._utilsLoaded) {
			var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
			scriptLoader.loadSubScript("chrome://calendar/content/calUtils.js");
			this._utilsLoaded = true;
		}

		return this.mFactory;
	},
	mFactory: {
		createInstance: function(outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmCalendarSync()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmCalendarSyncModule;
}
