/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;

function GenericWrappedXPComObject() {
	this.wrappedJSObject = this;
};
GenericWrappedXPComObject.prototype.QueryInterface = function(aIID) {
	if (!aIID.equals(Ci.nsISupports)) {
		throw Cr.NS_ERROR_NO_INTERFACE;
	}
	return this;
};

var stringBundle;

function getLocalizedMessage(aString) {
	dump("" + aString + "\n");
	try {
		return stringBundle.GetStringFromName("obm.sync." + aString);
	} catch(e) {
		return aString;
	}
};

var _max_script_run_time;
var _max_chrome_script_run_time;
function _disableTimeSecurity() {
	_max_script_run_time = UtilsOBM.getIntPref("dom.max_script_run_time", 10);
	_max_chrome_script_run_time = UtilsOBM.getIntPref("dom.max_chrome_script_run_time", 20);
	UtilsOBM.setIntPref("dom.max_script_run_time", 0);
	UtilsOBM.setIntPref("dom.max_chrome_script_run_time", 0);
	var globalData = Cc["@obm.org/globaldata;1"].getService().wrappedJSObject;
	globalData.addrSynchronizing = true;
};
function _restoreTimeSecurity() {
	UtilsOBM.setIntPref("dom.max_script_run_time", _max_script_run_time);
	UtilsOBM.setIntPref("dom.max_chrome_script_run_time", _max_chrome_script_run_time);
	var globalData = Cc["@obm.org/globaldata;1"].getService().wrappedJSObject;
	globalData.addrSynchronizing = false;
};

function ObmSyncManager() {
	dump("new ObmSyncManager\n");
	stringBundle = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService).createBundle("chrome://obm-extension/locale/calendar/errors_warnings.properties");
	this.abVersion = Cc["@mozilla.org/abmanager;1"] ? 3 : 2;
};

ObmSyncManager.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) && !aIID.equals(Ci.obmISyncManager)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	// Must be declared in the myComponent interface idl
	syncAll: function() {
		_disableTimeSecurity();
		var ObserverServ = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);

		var useRest = UtilsOBM.getBoolPref("extensions.obm.rest", false);
		if (useRest) {
			//use rest components
			var abSync = Cc["@obm.org/obm-sync-rest/addressbooksync;1"].createInstance(Ci.obmIRAddressBookSync);
			if (UtilsOBM.getBoolPref("extensions.obm.calEvents.is_sync_enabled", true) || UtilsOBM.getBoolPref("extensions.obm.calTasks.is_sync_enabled", true)) {
				var calSync = Cc["@obm.org/obm-sync-rest/calendarsync;1"].createInstance(Ci.obmIRCalendarSync);
			}
		} else {
			//use old soap components
			var abSync = Cc["@obm.org/obm-sync/addressbooksync;1"].createInstance(Ci.obmIAddressBookSync);
			if (UtilsOBM.getBoolPref("extensions.obm.calEvents.is_sync_enabled", true) || UtilsOBM.getBoolPref("extensions.obm.calTasks.is_sync_enabled", true)) {
				var calSync = Cc["@obm.org/obm-sync/calendarsync;1"].createInstance(Ci.obmICalendarSync);
				var catSync = Cc["@obm.org/obm-sync/categoriessync;1"].createInstance(Ci.obmICategoriesSync);
			}
		}

		var opIdx = 0;
		var operations = [];
		var i = 0;
		if (UtilsOBM.getBoolPref("extensions.obm.addressbooks.is_sync_enabled", true)) {
			operations[i++] = ["card",
			function(arg) {
				abSync.mergeAddressBook(arg)
			}];
		}
		if (UtilsOBM.getBoolPref("extensions.obm.calEvents.is_sync_enabled", true) || UtilsOBM.getBoolPref("extensions.obm.calTasks.is_sync_enabled", true)) {
			if (!useRest) {
				operations[i++] = ["calCat",
				function(arg) {
					catSync.mergeCategories(arg)
				}];
			}
			operations[i++] = ["cal",
			function(arg) {
				calSync.mergeAllCalendars(arg)
			}];
		}

		var self = this;
		var listener = {
			onStart: function() {},

			onNewOperation: function(aOpName, aOpData) {
				var msg = new GenericWrappedXPComObject();
				msg.opName = aOpName;
				msg.opData = aOpData;
				ObserverServ.notifyObservers(msg, "verbose-obm-sync-progress", "newOperation");
			},
			onOperationDone: function(aSuccess, aResultData) {
				var msg = new GenericWrappedXPComObject();
				msg.opRes = aSuccess;
				ObserverServ.notifyObservers(msg, "verbose-obm-sync-progress", "operationDone");
			},
			onProgress: function(aIdx, aTotal, aType, aEntity) {
				var msg = new GenericWrappedXPComObject();
				msg.idx = aIdx + 1;
				msg.total = aTotal;
				msg.type = aType;
				msg.entity = aEntity;
				ObserverServ.notifyObservers(msg, "verbose-obm-sync-progress", "operationProgress");
			},
			onError: function(aMessage) {
				dump("EXCEPTION: " + aMessage + "\n");
				var msg = new GenericWrappedXPComObject();
				msg.message = aMessage;
				ObserverServ.notifyObservers(msg, "verbose-obm-sync-progress", "operationSetError");
			},
			onStop: function(aRes, aMessage) {
				var msg = new GenericWrappedXPComObject();
				msg.message = aMessage;
				if (aRes == Components.results.NS_OK) {
					ObserverServ.notifyObservers(msg, "verbose-obm-sync-progress", "operationSetDone");
					this.nextOperation();
				} else {
					ObserverServ.notifyObservers(msg, "verbose-obm-sync-progress", "operationSetErrorFatal");
					_restoreTimeSecurity();
				}

				if (self.abVersion != 3) {
					var windowManager = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
					var addressBookWindow = windowManager.getMostRecentWindow("mail:addressbook");
					if (addressBookWindow) {
						var view = addressBookWindow.document.getElementById("dirTree").view;
						if (view) view.selection.select(0); // view.selection.currentIndex);
					}
				}
			},
			nextOperation: function() {
				if (opIdx >= operations.length) {
					ObserverServ.notifyObservers(null, "verbose-obm-sync-progress", "syncDone");
					_restoreTimeSecurity();
					return;
				}

				var msg = new GenericWrappedXPComObject();
				msg.opSetName = operations[opIdx][0];
				ObserverServ.notifyObservers(msg, "verbose-obm-sync-progress", "newOperationSet");
				try {
					operations[opIdx++][1](this);
				} catch(e) {
					listener.onError(e);
				}
			}
		};
		listener.nextOperation();
		return;
	},

	syncAddressBook: function() {
		_disableTimeSecurity();
		var windowsMediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var windows = windowsMediator.getEnumerator("mail:3pane");
		var window;
		if (windows.hasMoreElements()) {
			window = windows.getNext();
			var window = window.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIWebNavigation).QueryInterface(Ci.nsIDocShellTreeItem).treeOwner.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIXULWindow).XULBrowserWindow;
		}
		window.QueryInterface(Ci.nsIXULBrowserWindow);

		var useRest = UtilsOBM.getBoolPref("extensions.obm.rest", false);
		if (useRest) {
			//use rest component
			var abSync = Cc["@obm.org/obm-sync-rest/addressbooksync;1"].createInstance(Ci.obmIRAddressBookSync);
		} else {
			//use old soap component
			var abSync = Cc["@obm.org/obm-sync/addressbooksync;1"].createInstance(Ci.obmIAddressBookSync);
		}
		var opIdx = 0;
		var self = this;
		var operations = [];
		if (UtilsOBM.getBoolPref("extensions.obm.addressbooks.is_sync_enabled", true)) {
			operations[0] = ["AddressBook",
			function(arg) {
				abSync.mergeAddressBook(arg)
			}];
		}

		var self = this;
		var listener = {
			onStart: function() {},

			onNewOperation: function(aOpName) {
				var opName = null;
				switch (aOpName) {
				case "logUserIn":
					opName = "login";
					break;
				case "getSync":
					opName = "checking";
					break;
				case "mergeRemoteDiff":
					opName = "merging";
					break;
				case "sendLocalDiff":
					opName = "sending";
					break;
				case "listCategories":
					opName = "merging";
					break;
				default:
				}
				if (opName) window.setJSStatus(this.head + getLocalizedMessage(opName) + " ...");
			},
			onOperationDone: function(aResult) {},
			onProgress: function(aIdx, aTotal, aType, aEntity) {},
			onError: function(aMessage) {
				window.setJSStatus(this.head + aMessage);
			},
			onStop: function(aRes, aMessage) {
				if (aRes == Components.results.NS_OK) {
					window.setJSStatus(this.head + "ok");
					this.nextOperation();
				} else {
					window.setJSStatus(this.head + aMessage);
					_restoreTimeSecurity();
				}

				if (self.abVersion != 3) {
					var windowManager = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
					var addressBookWindow = windowManager.getMostRecentWindow("mail:addressbook");
					if (addressBookWindow) {
						var view = addressBookWindow.document.getElementById("dirTree").view;
						if (view) view.selection.select(
						/* view.selection.currentIndex */
						0);
					}
				}
			},
			nextOperation: function() {
				if (opIdx >= operations.length) {
					window.setJSStatus("OBM Sync - " + getLocalizedMessage("done"));
					_restoreTimeSecurity();
					return;
				}
				this.head = "OBM Sync - " + getLocalizedMessage("AddressBook") + ": ";
				try {
					operations[opIdx++][1](this);
				} catch(e) {
					listener.onError(e);
				}
			}
		};
		listener.nextOperation();
		return;
	},
	syncCalendar: function(aCalendar) {
		var windowsMediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var windows = windowsMediator.getEnumerator("mail:3pane");
		var window;
		if (windows.hasMoreElements()) {
			window = windows.getNext();
			var window = window.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIWebNavigation).QueryInterface(Ci.nsIDocShellTreeItem).treeOwner.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIXULWindow).XULBrowserWindow;
			if (!window) return;
		}
		window.QueryInterface(Ci.nsIXULBrowserWindow);
		
		_disableTimeSecurity();
		
		var useRest = UtilsOBM.getBoolPref("extensions.obm.rest", false);
		if (useRest) {
			var calSync = Cc["@obm.org/obm-sync-rest/calendarsync;1"].createInstance(Ci.obmIRCalendarSync);
		} else {
			var calSync = Cc["@obm.org/obm-sync/calendarsync;1"].createInstance(Ci.obmICalendarSync);
			var catSync = Cc["@obm.org/obm-sync/categoriessync;1"].createInstance(Ci.obmICategoriesSync);
		}

		var opIdx = 0;
		var self = aCalendar;
		var i = 0;
		var operations = [];
		if (!useRest) {
			operations[i++] = ["Categories",
			function(arg) {
				catSync.mergeCategories(arg)
			}];
		}
		operations[i++] = ["Calendar",
		function(arg) {
			calSync.mergeOneCalendar(self, arg)
		}];

		var listener = {
			onStart: function() {},

			onNewOperation: function(aOpName) {
				var opName = null;
				switch (aOpName) {
				case "logUserIn":
					opName = "login";
					break;
				case "getSync":
					opName = "checking";
					break;
				case "mergeRemoteDiff":
					opName = "merging";
					break;
				case "sendLocalDiff":
					opName = "sending";
					break;
				case "listCategories":
					opName = "merging";
					break;
				default:
				}
				if (opName) window.setJSStatus(this.head + getLocalizedMessage(opName) + " ...");
			},
			onOperationDone: function(aResult) {},
			onProgress: function(aIdx, aTotal, aType, aEntity) {},
			onError: function(aMessage) {
				window.setJSStatus(this.head + aMessage);
			},
			onStop: function(aRes, aMessage) {
				if (aRes == Components.results.NS_OK) {
					window.setJSStatus(this.head + "ok");
					this.nextOperation();
				} else {
					window.setJSStatus(this.head + aMessage);
					_restoreTimeSecurity();
				}
			},
			nextOperation: function() {
				if (opIdx >= operations.length) {
					window.setJSStatus("OBM Sync - " + getLocalizedMessage("done"));
					_restoreTimeSecurity();
					return;
				}
				this.head = "OBM Sync - " + getLocalizedMessage("Calendar") + ": ";
				try {
					operations[opIdx++][1](this);
				} catch(e) {
					listener.onError(e);
				}
			}
		};
		listener.nextOperation();
		return;
	}
};

var ObmSyncManagerModule = {
	mCID: Components.ID("{e37931ed-104f-4cf1-9b5c-73fab2d65424}"),
	mContractID: "@obm.org/syncmanager;1",
	registerSelf: function(compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.mCID, "OBM Sync Manager", this.mContractID, fileSpec, location, type);
	},
	getClassObject: function(compMgr, cid, iid) {

		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}
		if (!this._utilsLoaded) {
			var scriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader);
			scriptLoader.loadSubScript("chrome://obm-extension/content/calendar/UtilsOBM.js");
			this._utilsLoaded = true;
		}

		return this.mFactory;
	},
	mFactory: {
		createInstance: function(outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmSyncManager()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmSyncManagerModule;
}